from datetime import datetime
from unittest.mock import MagicMock, patch

import pytest

from foxglove_data_platform.client import Client

from .generate import generate_ros1_data


def test_download_without_decoder():
    with patch(
        "foxglove_data_platform.client.decoder_for_schema_encoding",
        MagicMock(side_effect=Exception("Not found!")),
    ):
        client = Client("test")
        client.download_data = MagicMock()
        client.download_data.return_value = generate_ros1_data()
        with pytest.raises(Exception):
            client.get_messages(
                device_id="test_id", start=datetime.now(), end=datetime.now()
            )


def test_download_with_decoder():
    client = Client("test")
    client.download_data = MagicMock()
    client.download_data.return_value = generate_ros1_data()
    messages = client.get_messages(
        device_id="test_id", start=datetime.now(), end=datetime.now()
    )
    assert len(messages) == 10



from datetime import datetime

import responses
from faker import Faker
from foxglove_data_platform.client import Client

from .api_url import api_url

fake = Faker()


@responses.activate
def test_create_device():
    id = fake.uuid4()
    name = "name"
    responses.add(
        responses.POST,
        api_url("/v1/devices"),
        json={
            "id": id,
            "name": name,
        },
    )
    client = Client("test")
    device = client.create_device(name=name)
    assert device["name"] == name


@responses.activate
def test_get_device():
    id = fake.uuid4()
    name = "name"
    responses.add(
        responses.GET,
        api_url(f"/v1/devices/{id}"),
        json={
            "id": id,
            "name": fake.sentence(2),
            "createdAt": datetime.now().isoformat(),
            "updatedAt": datetime.now().isoformat(),
        },
    )
    client = Client("test")
    device = client.get_device(device_id=id)
    assert device["id"] == id

    responses.add(
        responses.GET,
        api_url(f"/v1/devices/{name}"),
        json={
            "id": id,
            "name": fake.sentence(2),
            "createdAt": datetime.now().isoformat(),
            "updatedAt": datetime.now().isoformat(),
        },
    )
    device = client.get_device(device_name=name)
    assert device["id"] == id


@responses.activate
def test_get_devices():
    id = fake.uuid4()
    responses.add(
        responses.GET,
        api_url("/v1/devices"),
        json=[
            {
                "id": id,
                "name": fake.sentence(2),
                "createdAt": datetime.now().isoformat(),
                "updatedAt": datetime.now().isoformat(),
            }
        ],
    )
    client = Client("test")
    devices = client.get_devices()
    assert len(devices) == 1
    assert devices[0]["id"] == id


@responses.activate
def test_delete_device():
    id = fake.uuid4()
    name = "name"
    responses.add(
        responses.DELETE,
        api_url(f"/v1/devices/{id}"),
        json={"success": True},
    )
    client = Client("test")
    try:
        client.delete_device(device_id=id)
    except:
        assert False

    responses.add(
        responses.DELETE,
        api_url(f"/v1/devices/{name}"),
        json={"success": True},
    )
    try:
        client.delete_device(device_name=name)
    except:
        assert False



from datetime import datetime

import responses
from faker import Faker
from foxglove_data_platform.client import Client

from .api_url import api_url

fake = Faker()


@responses.activate
def test_get_coverage():
    device_id = fake.uuid4()
    device_name = fake.name()
    responses.add(
        responses.GET,
        api_url(f"/v1/data/coverage"),
        json=[
            {
                "deviceId": device_id,
                "device": {"id": device_id, "name": device_name},
                "start": datetime.now().isoformat(),
                "end": datetime.now().isoformat(),
            },
            {
                "deviceId": device_id,
                "device": {"id": device_id, "name": device_name},
                "start": datetime.now().isoformat(),
                "end": datetime.now().isoformat(),
            },
        ],
    )
    client = Client("test")
    coverage = client.get_coverage(
        start=datetime.now(), end=datetime.now(), device_id=device_id
    )
    assert len(coverage) == 2
    assert coverage[0]["device_id"] == device_id



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: string_message.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x14string_message.proto"\x1d\n\rStringMessage\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\tb\x06proto3'
)


_STRINGMESSAGE = DESCRIPTOR.message_types_by_name["StringMessage"]
StringMessage = _reflection.GeneratedProtocolMessageType(
    "StringMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _STRINGMESSAGE,
        "__module__": "string_message_pb2"
        # @@protoc_insertion_point(class_scope:StringMessage)
    },
)
_sym_db.RegisterMessage(StringMessage)

if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _STRINGMESSAGE._serialized_start = 24
    _STRINGMESSAGE._serialized_end = 53
# @@protoc_insertion_point(module_scope)



import responses
from faker import Faker
from foxglove_data_platform.client import Client
from datetime import datetime
from dateutil.tz import tzoffset

from .api_url import api_url

fake = Faker()


@responses.activate
def test_get_attachments():
    device_id = fake.uuid4()
    device_name = fake.name()
    attachment_id = fake.uuid4()
    recording_id = fake.uuid4()
    path = fake.file_name(extension="txt")
    media_type = fake.mime_type()
    size = fake.random_number()
    crc = fake.random_number()
    fingerprint = fake.uuid4()
    site_id = fake.uuid4()
    now = datetime.now(tzoffset(None, 0))

    responses.add(
        responses.GET,
        api_url(f"/v1/recording-attachments"),
        json=[
            {
                "id": attachment_id,
                "recordingId": recording_id,
                "device": {"id": device_id, "name": device_name},
                "siteId": site_id,
                "name": path,
                "mediaType": media_type,
                "size": size,
                "crc": crc,
                "fingerprint": fingerprint,
                "logTime": now.isoformat(),
                "createTime": now.isoformat(),
            },
        ],
        match=[responses.matchers.query_string_matcher("sortBy=logTime")],
    )
    client = Client("test")
    attachments = client.get_attachments(
        sort_by="log_time",
    )
    assert attachments == [
        {
            "id": attachment_id,
            "recording_id": recording_id,
            "site_id": site_id,
            "name": path,
            "device": {
                "id": device_id,
                "name": device_name,
            },
            "media_type": media_type,
            "size": size,
            "crc": crc,
            "fingerprint": fingerprint,
            "log_time": now,
            "create_time": now,
        },
    ]


@responses.activate
def test_download_attachment():
    id = "abcde"
    data = fake.binary(4096)
    responses.add(
        responses.GET,
        api_url(f"/v1/recording-attachments/{id}/download"),
        body=data,
    )
    client = Client("test")
    response_data = client.download_attachment(id=id)
    assert data == response_data



from datetime import datetime
from unittest.mock import MagicMock

from foxglove_data_platform.client import Client

from .generate import generate_json_data


def test_download_with_decoder():
    client = Client("test")
    client.download_data = MagicMock(return_value=generate_json_data())
    messages = client.get_messages(
        device_id="test_id", start=datetime.now(), end=datetime.now()
    )
    assert len(messages) == 10
    for _, _, msg in messages:
        assert "level" in msg



from datetime import datetime

import responses
from faker import Faker
from foxglove_data_platform.client import Client

from .api_url import api_url

fake = Faker()


@responses.activate
def test_delete_import():
    import_id = fake.uuid4()
    responses.add(
        responses.DELETE,
        api_url(f"/v1/data/imports/{import_id}"),
        json={"success": True},
    )
    try:
        client = Client("test")
        client.delete_import(import_id=import_id)
    except:
        assert False


@responses.activate
def test_get_imports():
    device_id = "my_device_id"
    import_id = "my_device_id"
    responses.add(
        responses.GET,
        api_url(f"/v1/data/imports"),
        json=[
            {
                "importId": import_id,
                "deviceId": device_id,
                "importTime": datetime.now().isoformat(),
                "start": datetime.now().isoformat(),
                "end": datetime.now().isoformat(),
                "metadata": {},
                "inputType": "bag",
                "outputType": "mcap0",
                "filename": "test.bag",
                "inputSize": 1024,
                "totalOutputSize": 1024,
            }
        ],
    )
    client = Client("test")
    imports = client.get_imports(device_id=device_id)
    assert len(imports) == 1
    assert imports[0]["device_id"] == device_id


@responses.activate
def test_get_deleted_imports():
    responses.add(
        responses.GET,
        api_url(f"/v1/data/imports"),
        json=[],
    )
    responses.add(
        responses.GET,
        api_url(f"/v1/data/imports"),
        json=[
            {
                "importId": "import_id",
                "deviceId": "device_id",
                "importTime": datetime.now().isoformat(),
                "start": datetime.now().isoformat(),
                "end": datetime.now().isoformat(),
                "metadata": {},
                "inputType": "bag",
                "outputType": "mcap0",
                "filename": "test.bag",
                "inputSize": 1024,
                "totalOutputSize": 1024,
            }
        ],
        match=[responses.matchers.query_string_matcher("includeDeleted=true")],
    )
    client = Client("test")
    imports = client.get_imports()
    deleted_imports = client.get_imports(include_deleted=True)
    assert len(imports) == 0
    assert len(deleted_imports) == 1



from pathlib import Path
import time
from io import BytesIO
import json

from mcap.writer import Writer
from mcap_ros2.writer import Writer as Ros2Writer
from std_msgs.msg import String  # type: ignore
from .string_message_pb2 import StringMessage


def generate_ros1_data():
    output = BytesIO()
    writer = Writer(output)
    writer.start(profile="ros1", library="test")
    string_schema_id = writer.register_schema(
        name=String._type, encoding="ros1msg", data=String._full_text.encode()  # type: ignore
    )
    string_channel_id = writer.register_channel(
        topic="chatter", message_encoding="ros1", schema_id=string_schema_id
    )

    for i in range(1, 11):
        s = String(data=f"string message {i}")
        buff = BytesIO()
        s.serialize(buff)  # type: ignore
        writer.add_message(
            channel_id=string_channel_id,
            log_time=time.time_ns(),
            data=buff.getvalue(),
            publish_time=time.time_ns(),
        )
    writer.finish()
    return output.getvalue()


def generate_ros2_data():
    output = BytesIO()
    with Ros2Writer(output) as writer:
        schema = writer.register_msgdef(String._type, String._full_text)

        for i in range(1, 11):
            writer.write_message("/chatter", schema, {"data": f"string message {i}"})
    return output.getvalue()


def generate_json_data():
    output = BytesIO()
    mcap_writer = Writer(output)
    mcap_writer.start(profile="", library="test")
    schema_id = mcap_writer.register_schema(
        name="mood",
        encoding="jsonschema",
        data=json.dumps(
            {
                "type": "object",
                "properties": {
                    "happy": {"type": "boolean"},
                    "level": {"type": "number"},
                },
            }
        ).encode("utf-8"),
    )
    channel_id = mcap_writer.register_channel(
        topic="moods",
        message_encoding="json",
        schema_id=schema_id,
    )
    for i in range(1, 11):
        m = json.dumps({"happy": True, "level": i}).encode("utf-8")
        mcap_writer.add_message(
            channel_id=channel_id,
            log_time=time.time_ns(),
            data=m,
            publish_time=time.time_ns(),
        )
    mcap_writer.finish()
    return output.getvalue()


def generate_protobuf_data():
    output = BytesIO()
    mcap_writer = Writer(output)
    mcap_writer.start(profile="protobuf", library="test")

    schema_id = mcap_writer.register_schema(
        name="StringMessage",
        encoding="protobuf",
        data=(Path(__file__).parent / "string_message.fds").read_bytes(),
    )
    channel_id = mcap_writer.register_channel(
        topic="chatter", message_encoding="protobuf", schema_id=schema_id
    )

    for i in range(1, 11):
        s = StringMessage(data=f"string message {i}")
        mcap_writer.add_message(
            channel_id=channel_id,
            log_time=time.time_ns(),
            data=s.SerializeToString(),  # type: ignore
            publish_time=time.time_ns(),
        )
    mcap_writer.finish()
    return output.getvalue()
